from enum import Enum

class OAuthProvider(Enum):
    AMAZON = "amazon"
    APPLE = "apple"
    AUTH0 = "auth0"
    AUTHENTIK = "authentik"
    AUTODESK = "autodesk"
    BITBUCKET = "bitbucket"
    BITLY = "bitly"
    BOX = "box"
    DAILYMOTION = "dailymotion"
    DISCORD = "discord"
    DISQUS = "disqus"
    DROPBOX = "dropbox"
    ETSY = "etsy"
    FACEBOOK = "facebook"
    FIGMA = "figma"
    GITHUB = "github"
    GITLAB = "gitlab"
    GOOGLE = "google"
    LINKEDIN = "linkedin"
    MICROSOFT = "microsoft"
    NOTION = "notion"
    OIDC = "oidc"
    OKTA = "okta"
    PAYPAL = "paypal"
    PAYPALSANDBOX = "paypalSandbox"
    PODIO = "podio"
    SALESFORCE = "salesforce"
    SLACK = "slack"
    SPOTIFY = "spotify"
    STRIPE = "stripe"
    TRADESHIFT = "tradeshift"
    TRADESHIFTBOX = "tradeshiftBox"
    TWITCH = "twitch"
    WORDPRESS = "wordpress"
    YAHOO = "yahoo"
    YAMMER = "yammer"
    YANDEX = "yandex"
    ZOHO = "zoho"
    ZOOM = "zoom"
    MOCK = "mock"
