from enum import Enum

class Framework(Enum):
    ANALOG = "analog"
    ANGULAR = "angular"
    NEXTJS = "nextjs"
    REACT = "react"
    NUXT = "nuxt"
    VUE = "vue"
    SVELTEKIT = "sveltekit"
    ASTRO = "astro"
    REMIX = "remix"
    LYNX = "lynx"
    FLUTTER = "flutter"
    REACT_NATIVE = "react-native"
    VITE = "vite"
    OTHER = "other"
