from enum import Enum

class Name(Enum):
    V1_DATABASE = "v1-database"
    V1_DELETES = "v1-deletes"
    V1_AUDITS = "v1-audits"
    V1_MAILS = "v1-mails"
    V1_FUNCTIONS = "v1-functions"
    V1_STATS_RESOURCES = "v1-stats-resources"
    V1_STATS_USAGE = "v1-stats-usage"
    V1_WEBHOOKS = "v1-webhooks"
    V1_CERTIFICATES = "v1-certificates"
    V1_BUILDS = "v1-builds"
    V1_MESSAGING = "v1-messaging"
    V1_MIGRATIONS = "v1-migrations"
