from enum import Enum

class Browser(Enum):
    AVANT_BROWSER = "aa"
    ANDROID_WEBVIEW_BETA = "an"
    GOOGLE_CHROME = "ch"
    GOOGLE_CHROME_IOS = "ci"
    GOOGLE_CHROME_MOBILE = "cm"
    CHROMIUM = "cr"
    MOZILLA_FIREFOX = "ff"
    SAFARI = "sf"
    MOBILE_SAFARI = "mf"
    MICROSOFT_EDGE = "ps"
    MICROSOFT_EDGE_IOS = "oi"
    OPERA_MINI = "om"
    OPERA = "op"
    OPERA_NEXT = "on"
