from enum import Enum

class CreditCard(Enum):
    AMERICAN_EXPRESS = "amex"
    ARGENCARD = "argencard"
    CABAL = "cabal"
    CENCOSUD = "cencosud"
    DINERS_CLUB = "diners"
    DISCOVER = "discover"
    ELO = "elo"
    HIPERCARD = "hipercard"
    JCB = "jcb"
    MASTERCARD = "mastercard"
    NARANJA = "naranja"
    TARJETA_SHOPPING = "targeta-shopping"
    UNION_CHINA_PAY = "union-china-pay"
    VISA = "visa"
    MIR = "mir"
    MAESTRO = "maestro"
    RUPAY = "rupay"
