from enum import Enum

class Flag(Enum):
    AFGHANISTAN = "af"
    ANGOLA = "ao"
    ALBANIA = "al"
    ANDORRA = "ad"
    UNITED_ARAB_EMIRATES = "ae"
    ARGENTINA = "ar"
    ARMENIA = "am"
    ANTIGUA_AND_BARBUDA = "ag"
    AUSTRALIA = "au"
    AUSTRIA = "at"
    AZERBAIJAN = "az"
    BURUNDI = "bi"
    BELGIUM = "be"
    BENIN = "bj"
    BURKINA_FASO = "bf"
    BANGLADESH = "bd"
    BULGARIA = "bg"
    BAHRAIN = "bh"
    BAHAMAS = "bs"
    BOSNIA_AND_HERZEGOVINA = "ba"
    BELARUS = "by"
    BELIZE = "bz"
    BOLIVIA = "bo"
    BRAZIL = "br"
    BARBADOS = "bb"
    BRUNEI_DARUSSALAM = "bn"
    BHUTAN = "bt"
    BOTSWANA = "bw"
    CENTRAL_AFRICAN_REPUBLIC = "cf"
    CANADA = "ca"
    SWITZERLAND = "ch"
    CHILE = "cl"
    CHINA = "cn"
    COTE_DIVOIRE = "ci"
    CAMEROON = "cm"
    DEMOCRATIC_REPUBLIC_OF_THE_CONGO = "cd"
    REPUBLIC_OF_THE_CONGO = "cg"
    COLOMBIA = "co"
    COMOROS = "km"
    CAPE_VERDE = "cv"
    COSTA_RICA = "cr"
    CUBA = "cu"
    CYPRUS = "cy"
    CZECH_REPUBLIC = "cz"
    GERMANY = "de"
    DJIBOUTI = "dj"
    DOMINICA = "dm"
    DENMARK = "dk"
    DOMINICAN_REPUBLIC = "do"
    ALGERIA = "dz"
    ECUADOR = "ec"
    EGYPT = "eg"
    ERITREA = "er"
    SPAIN = "es"
    ESTONIA = "ee"
    ETHIOPIA = "et"
    FINLAND = "fi"
    FIJI = "fj"
    FRANCE = "fr"
    MICRONESIA_FEDERATED_STATES_OF = "fm"
    GABON = "ga"
    UNITED_KINGDOM = "gb"
    GEORGIA = "ge"
    GHANA = "gh"
    GUINEA = "gn"
    GAMBIA = "gm"
    GUINEA_BISSAU = "gw"
    EQUATORIAL_GUINEA = "gq"
    GREECE = "gr"
    GRENADA = "gd"
    GUATEMALA = "gt"
    GUYANA = "gy"
    HONDURAS = "hn"
    CROATIA = "hr"
    HAITI = "ht"
    HUNGARY = "hu"
    INDONESIA = "id"
    INDIA = "in"
    IRELAND = "ie"
    IRAN_ISLAMIC_REPUBLIC_OF = "ir"
    IRAQ = "iq"
    ICELAND = "is"
    ISRAEL = "il"
    ITALY = "it"
    JAMAICA = "jm"
    JORDAN = "jo"
    JAPAN = "jp"
    KAZAKHSTAN = "kz"
    KENYA = "ke"
    KYRGYZSTAN = "kg"
    CAMBODIA = "kh"
    KIRIBATI = "ki"
    SAINT_KITTS_AND_NEVIS = "kn"
    SOUTH_KOREA = "kr"
    KUWAIT = "kw"
    LAO_PEOPLES_DEMOCRATIC_REPUBLIC = "la"
    LEBANON = "lb"
    LIBERIA = "lr"
    LIBYA = "ly"
    SAINT_LUCIA = "lc"
    LIECHTENSTEIN = "li"
    SRI_LANKA = "lk"
    LESOTHO = "ls"
    LITHUANIA = "lt"
    LUXEMBOURG = "lu"
    LATVIA = "lv"
    MOROCCO = "ma"
    MONACO = "mc"
    MOLDOVA = "md"
    MADAGASCAR = "mg"
    MALDIVES = "mv"
    MEXICO = "mx"
    MARSHALL_ISLANDS = "mh"
    NORTH_MACEDONIA = "mk"
    MALI = "ml"
    MALTA = "mt"
    MYANMAR = "mm"
    MONTENEGRO = "me"
    MONGOLIA = "mn"
    MOZAMBIQUE = "mz"
    MAURITANIA = "mr"
    MAURITIUS = "mu"
    MALAWI = "mw"
    MALAYSIA = "my"
    NAMIBIA = "na"
    NIGER = "ne"
    NIGERIA = "ng"
    NICARAGUA = "ni"
    NETHERLANDS = "nl"
    NORWAY = "no"
    NEPAL = "np"
    NAURU = "nr"
    NEW_ZEALAND = "nz"
    OMAN = "om"
    PAKISTAN = "pk"
    PANAMA = "pa"
    PERU = "pe"
    PHILIPPINES = "ph"
    PALAU = "pw"
    PAPUA_NEW_GUINEA = "pg"
    POLAND = "pl"
    FRENCH_POLYNESIA = "pf"
    NORTH_KOREA = "kp"
    PORTUGAL = "pt"
    PARAGUAY = "py"
    QATAR = "qa"
    ROMANIA = "ro"
    RUSSIA = "ru"
    RWANDA = "rw"
    SAUDI_ARABIA = "sa"
    SUDAN = "sd"
    SENEGAL = "sn"
    SINGAPORE = "sg"
    SOLOMON_ISLANDS = "sb"
    SIERRA_LEONE = "sl"
    EL_SALVADOR = "sv"
    SAN_MARINO = "sm"
    SOMALIA = "so"
    SERBIA = "rs"
    SOUTH_SUDAN = "ss"
    SAO_TOME_AND_PRINCIPE = "st"
    SURINAME = "sr"
    SLOVAKIA = "sk"
    SLOVENIA = "si"
    SWEDEN = "se"
    ESWATINI = "sz"
    SEYCHELLES = "sc"
    SYRIA = "sy"
    CHAD = "td"
    TOGO = "tg"
    THAILAND = "th"
    TAJIKISTAN = "tj"
    TURKMENISTAN = "tm"
    TIMOR_LESTE = "tl"
    TONGA = "to"
    TRINIDAD_AND_TOBAGO = "tt"
    TUNISIA = "tn"
    TURKEY = "tr"
    TUVALU = "tv"
    TANZANIA = "tz"
    UGANDA = "ug"
    UKRAINE = "ua"
    URUGUAY = "uy"
    UNITED_STATES = "us"
    UZBEKISTAN = "uz"
    VATICAN_CITY = "va"
    SAINT_VINCENT_AND_THE_GRENADINES = "vc"
    VENEZUELA = "ve"
    VIETNAM = "vn"
    VANUATU = "vu"
    SAMOA = "ws"
    YEMEN = "ye"
    SOUTH_AFRICA = "za"
    ZAMBIA = "zm"
    ZIMBABWE = "zw"
